#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_host_generic.h"
#include "user.h"
#include "timer.h"

#define DEBUG_MODE
// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************

        _CONFIG1(WDTPS_PS1 & FWPSA_PR32 & WINDIS_OFF & FWDTEN_OFF & ICS_PGx1 & GWRP_OFF & GCP_OFF & JTAGEN_OFF)
        _CONFIG2(POSCMOD_HS & I2C1SEL_PRI & IOL1WAY_OFF & OSCIOFNC_ON & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_NODIV & IESO_ON)
        _CONFIG3(WPFP_WPFP0 & SOSCSEL_SOSC & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
        _CONFIG4(DSWDTPS_DSWDTPS3 & DSWDTOSC_LPRC & RTCOSC_SOSC & DSBOREN_OFF & DSWDTEN_OFF)
 
// *****************************************************************************

// Application States
typedef enum
{
    BT_INITIALIZE = 0,                // Initialize the app when a device is attached
    BT_STATE_IDLE,                    // Inactive State
    BT_STATE_PROCESS,
	BT_STATE_WRITE_CLASS,
	BT_STATE_READ_EP1,
	BT_STATE_READ_CLASS_WAITING,
	BT_STATE_READ_ACL,
	BT_STATE_WRITE_ACL,
	BT_STATE_READ_ACL_WAITING,
    BT_STATE_ERROR                    // An error has occured
} BT_STATE;

// Hci States
typedef enum
{
	HCI_CMD_RESET = 0,                // Initialize the hci when a device is attached
		HCI_CMD_RESET_WRITE_END,
		HCI_CMD_RESET_READ_END,
	HCI_CMD_READ_BD_ADDR,
		HCI_CMD_READ_BD_ADDR_WRITE_END,
		HCI_CMD_READ_BD_ADDR_READ_END,
	HCI_CMD_LOCAL_NAME,
		HCI_CMD_LOCAL_NAME_WRITE_END,
		HCI_CMD_LOCAL_NAME_READ_END,
	HCI_CMD_CLASS_DEVICE,
		HCI_CMD_CLASS_DEVICE_WRITE_END,
		HCI_CMD_CLASS_DEVICE_READ_END,
	HCI_CMD_SCAN_ENABLE,
		HCI_CMD_SCAN_ENABLE_WRITE_END,
		HCI_CMD_SCAN_ENABLE_READ_END,
	HCI_CMD_WAIT_CONNECTION,
		HCI_CMD_WAIT_CONNECTION_READ_END,
	HCI_CMD_CONNECTION_ACCEPT,
		HCI_CMD_CONNECTION_ACCEPT_WRITE_END,
		HCI_CMD_CONNECTION_ACCEPT_READ_END,

	L2CAP_CON_REQ,
		L2CAP_CON_REQ_READ_END,
	L2CAP_CON_RESP,
	L2CAP_CONFIG_REQ,
		L2CAP_CONFIG_REQ_READ_END,
	L2CAP_CONFIG_RESP,
	L2CAP_CONFIG_REQ_HOST,
		L2CAP_CONFIG_RESP_HOST,
		L2CAP_CONFIG_REQ_HOST_READ_END,

	SDP_SEARCH_REQ,
	SDP_SEARCH_REQ_READ_END,
	SDP_SERCH_RESP,
	SDP_SERCH_RESP_MICRO,
	SDP_ATTR_REQ,
		SDP_ATTR_REQ_READ_END,
	SDP_ATTR_RESP2a,
	SDP_ATTR_RESP2a_READ,
		SDP_ATTR_RESP2a_READ_END,
	SDP_ATTR_RESP2b,
	SDP_ATTR_RESP2b_READ,
		SDP_ATTR_RESP2b_READ_END,
	SDP_ATTR_RESP2c,
		SDP_ATTR_RESP2c_READ,
		SDP_ATTR_RESP2c_READ_END,
	SDP_ATTR_RESP2d,
		SDP_ATTR_RESP2d_READ,
		SDP_ATTR_RESP2d_READ_END,

	SDP_ATTR_RESP4a,
	SDP_ATTR_RESP4a_READ,
		SDP_ATTR_RESP4a_READ_END,
	SDP_ATTR_RESP4b,
	SDP_ATTR_RESP4b_READ,
		SDP_ATTR_RESP4b_READ_END,
	SDP_ATTR_RESP4c,
		SDP_ATTR_RESP4c_READ,
		SDP_ATTR_RESP4c_READ_END,
	SDP_ATTR_RESP4d,
		SDP_ATTR_RESP4d_READ,
		SDP_ATTR_RESP4d_READ_END,
	SDP_ATTR_RESP4e,
		SDP_ATTR_RESP4e_READ,
		SDP_ATTR_RESP4e_READ_END,
	SDP_ATTR_RESP4f,
		SDP_ATTR_RESP4f_READ,
		SDP_ATTR_RESP4f_READ_END,
	SDP_ATTR_RESP5,
		SDP_ATTR_RESP5_READ,
		SDP_ATTR_RESP5_READ_END,
	SDP_ATTR_END,

	HID_START,
		HID_START_RESP,
		HID_HANDSHAKE_ERR,
	HID_READ_DATA,
		HID_READ_DATA_END,
		HID_HANDSHAKE_SUCCESS,
	HID_WRITE_DATA,

	OUT1,
		OUT_END1,
	OUT2,
		OUT_END2,
	OUT3,
	OUT_END3,
	OUT4,
		OUT_END4,
	OUT5,
		OUT_END5,
	OUT7,
		OUT_END7,
	OUT7a,//a is for Microsoft 
		OUT_END7a,
	OUT8,
		OUT_END8,
	OUT8a,
		OUT_END8a,
	OUT9,
		OUT_END9,
	OUT9a,
		OUT_END9a,
	OUT10,
		OUT_END10,
	OUT10a,
		OUT_END10a,

	OUT11a,
		OUT_END11a,
	OUT12a,
		OUT_END12a,
	OUT13a,
		OUT_END13a,
	PROG_END
} HCI_STATE;


// *****************************************************************************
// *****************************************************************************
// Global Variables
// *****************************************************************************
// *****************************************************************************

BYTE	deviceAddress;  // Address of the device on the USB
BT_STATE  DemoState;      // Current state of the demo application
HCI_STATE  HciState;      // Current state of the demo application
WORD data_size;
int data_num;
unsigned char local_bd_addr[6];//PIC
unsigned char remote_bd_addr[6];//PC
unsigned char buf[64];
unsigned char buf1[64];
unsigned char handle[2];//a handle for ACL 
unsigned char src_cid[2];//HID interrupt (data) after hid_flag=2
unsigned char dst_cid[2];
unsigned char src_cid1[2];//HID control after hid_flag=3
unsigned char dst_cid1[2];
unsigned char hid_flag;
int pdata;
int step;
unsigned char delay_time=30;//see 4906 line from the top of usb_host.c This is set to be 0 after L2CAP layer.
// data for SDP

unsigned char att2a[5][31]=
{
{0x26,0x20,0x1b,0x00,0x80,0x00,0x46,0x00,0x05,0x00,0x01,0x00,0x7b,0x00,0x76,0x36,0x01,0xcc,0x09,0x00,0x00,0x0a,0x00,0x01,0x00,0x00,0x09,0x00,0x01,0x35,0x03},
{0x26,0x10,0x1b,0x00,0x19,0x11,0x24,0x09,0x00,0x04,0x35,0x0d,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x11,0x35,0x03,0x19,0x00,0x11,0x09,0x00,0x05,0x35,0x03,0x19},
{0x26,0x10,0x1b,0x00,0x10,0x02,0x09,0x00,0x06,0x35,0x09,0x09,0x65,0x6e,0x09,0x00,0x6a,0x09,0x01,0x00,0x09,0x00,0x09,0x35,0x08,0x35,0x06,0x19,0x11,0x24,0x09},
{0x26,0x10,0x1b,0x00,0x01,0x00,0x09,0x00,0x0d,0x35,0x0f,0x35,0x0d,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x13,0x35,0x03,0x19,0x00,0x11,0x09,0x01,0x00,0x25,0x13},
{0x26,0x10,0x18,0x00,0x4e,0x69,0x6e,0x74,0x65,0x6e,0x64,0x6f,0x20,0x52,0x56,0x4c,0x2d,0x43,0x4e,0x54,0x2d,0x30,0x31,0x09,0x01,0x02,0x00,0x76,0xff,0xff,0xff}
};
unsigned char att2b[5][31]=
{
{0x26,0x20,0x1b,0x00,0x80,0x00,0x46,0x00,0x05,0x00,0x02,0x00,0x7b,0x00,0x76,0x01,0x25,0x13,0x4e,0x69,0x6e,0x74,0x65,0x6e,0x64,0x6f,0x20,0x52,0x56,0x4c,0x2d},
{0x26,0x10,0x1b,0x00,0x43,0x4e,0x54,0x2d,0x30,0x31,0x09,0x01,0x02,0x25,0x08,0x4e,0x69,0x6e,0x74,0x65,0x6e,0x64,0x6f,0x09,0x02,0x00,0x09,0x01,0x00,0x09,0x02},
{0x26,0x10,0x1b,0x00,0x01,0x09,0x01,0x11,0x09,0x02,0x02,0x08,0x04,0x09,0x02,0x03,0x08,0x33,0x09,0x02,0x04,0x28,0x00,0x09,0x02,0x05,0x28,0x01,0x09,0x02,0x06},
{0x26,0x10,0x1b,0x00,0x35,0xdf,0x35,0xdd,0x08,0x22,0x25,0xd9,0x05,0x01,0x09,0x05,0xa1,0x01,0x85,0x10,0x15,0x00,0x26,0xff,0x00,0x75,0x08,0x95,0x01,0x06,0x00},
{0x26,0x10,0x18,0x00,0xff,0x09,0x01,0x91,0x00,0x85,0x11,0x95,0x01,0x09,0x01,0x91,0x00,0x85,0x12,0x95,0x02,0x09,0x01,0x91,0x00,0x02,0x00,0xec,0xff,0xff,0xff}
};
unsigned char att2c[5][31]=
{
{0x26,0x20,0x1b,0x00,0x80,0x00,0x46,0x00,0x05,0x00,0x01,0x00,0x7b,0x00,0x76,0x85,0x13,0x95,0x01,0x09,0x01,0x91,0x00,0x85,0x14,0x95,0x01,0x09,0x01,0x91,0x00},
{0x26,0x10,0x1b,0x00,0x85,0x15,0x95,0x01,0x09,0x01,0x91,0x00,0x85,0x16,0x95,0x15,0x09,0x01,0x91,0x00,0x85,0x17,0x95,0x06,0x09,0x01,0x91,0x00,0x85,0x18,0x95},
{0x26,0x10,0x1b,0x00,0x15,0x09,0x01,0x91,0x00,0x85,0x19,0x95,0x01,0x09,0x01,0x91,0x00,0x85,0x1a,0x95,0x01,0x09,0x01,0x91,0x00,0x85,0x20,0x95,0x06,0x09,0x01},
{0x26,0x10,0x1b,0x00,0x81,0x00,0x85,0x21,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x22,0x95,0x04,0x09,0x01,0x81,0x00,0x85,0x30,0x95,0x02,0x09,0x01,0x81,0x00,0x85},
{0x26,0x10,0x18,0x00,0x31,0x95,0x05,0x09,0x01,0x81,0x00,0x85,0x32,0x95,0x0a,0x09,0x01,0x81,0x00,0x85,0x33,0x95,0x11,0x09,0x01,0x02,0x01,0x62,0xff,0xff,0xff}
};

unsigned char att2d[5][31]=
{
{0x26,0x20,0x1b,0x00,0x75,0x00,0x46,0x00,0x05,0x00,0x01,0x00,0x70,0x00,0x6d,0x81,0x00,0x85,0x34,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x35,0x95,0x15,0x09,0x01},
{0x26,0x10,0x1b,0x00,0x81,0x00,0x85,0x36,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x37,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x3d,0x95,0x15,0x09,0x01,0x81,0x00,0x85},
{0x26,0x10,0x1b,0x00,0x3e,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x3f,0x95,0x15,0x09,0x01,0x81,0x00,0xc0,0x09,0x02,0x07,0x35,0x08,0x35,0x06,0x09,0x04,0x09,0x09},
{0x26,0x10,0x1b,0x00,0x01,0x00,0x09,0x02,0x08,0x28,0x00,0x09,0x02,0x09,0x28,0x01,0x09,0x02,0x0a,0x28,0x01,0x09,0x02,0x0b,0x09,0x01,0x00,0x09,0x02,0x0c,0x09},
{0x26,0x10,0x0d,0x00,0x0c,0x80,0x09,0x02,0x0d,0x28,0x00,0x09,0x02,0x0e,0x28,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff}
};

// for Microchip bluetooth stack
unsigned char att4a[5][31]=
{
{0x26,0x20,0x1B,0x00,0x80,0x00,0x40,0x00,0x07,0x00,0x00,0x00,0x7B,0x00,0x76,0x36,0x02,0x7C,0x36,0x00,0x4D,0x09,0x00,0x00,0x0A,0x00,0x00,0x00,0x00,0x09,0x00},
{0x26,0x10,0x1B,0x00,0x01,0x35,0x03,0x19,0x10,0x00,0x09,0x00,0x04,0x35,0x0D,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x01,0x35,0x03,0x19,0x00,0x01,0x09,0x00,0x05},
{0x26,0x10,0x1B,0x00,0x35,0x03,0x19,0x10,0x02,0x09,0x00,0x06,0x35,0x09,0x09,0x65,0x6E,0x09,0x00,0x6A,0x09,0x01,0x00,0x09,0x00,0x09,0x35,0x08,0x35,0x06,0x19},
{0x26,0x10,0x1B,0x00,0x01,0x00,0x09,0x01,0x00,0x09,0x02,0x00,0x35,0x03,0x09,0x01,0x00,0x36,0x01,0xCC,0x09,0x00,0x00,0x0A,0x00,0x01,0x00,0x00,0x09,0x00,0x01},
{0x26,0x10,0x18,0x00,0x35,0x03,0x19,0x11,0x24,0x09,0x00,0x04,0x35,0x0D,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x11,0x35,0x03,0x19,0x02,0x00,0x76,0xFF,0xFF,0xFF}
};

unsigned char att4b[5][31]=
{
{0x26,0x20,0x1B,0x00,0x80,0x00,0x40,0x00,0x07,0x00,0x01,0x00,0x7B,0x00,0x76,0x00,0x11,0x09,0x00,0x05,0x35,0x03,0x19,0x10,0x02,0x09,0x00,0x06,0x35,0x09,0x09},
{0x26,0x10,0x1B,0x00,0x65,0x6E,0x09,0x00,0x6A,0x09,0x01,0x00,0x09,0x00,0x09,0x35,0x08,0x35,0x06,0x19,0x11,0x24,0x09,0x01,0x00,0x09,0x00,0x0D,0x35,0x0F,0x35},
{0x26,0x10,0x1B,0x00,0x0D,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x13,0x35,0x03,0x19,0x00,0x11,0x09,0x01,0x00,0x25,0x13,0x4E,0x69,0x6E,0x74,0x65,0x6E,0x64,0x6F},
{0x26,0x10,0x1B,0x00,0x20,0x52,0x56,0x4C,0x2D,0x43,0x4E,0x54,0x2D,0x30,0x31,0x09,0x01,0x01,0x25,0x13,0x4E,0x69,0x6E,0x74,0x65,0x6E,0x64,0x6F,0x20,0x52,0x56},
{0x26,0x10,0x18,0x00,0x4C,0x2D,0x43,0x4E,0x54,0x2D,0x30,0x31,0x09,0x01,0x02,0x25,0x08,0x4E,0x69,0x6E,0x74,0x65,0x6E,0x64,0x6F,0x02,0x00,0xEC,0xFF,0xFF,0xFF}
};

unsigned char att4c[5][31]=
{
{0x26,0x20,0x1B,0x00,0x80,0x00,0x40,0x00,0x07,0x00,0x02,0x00,0x7B,0x00,0x76,0x09,0x02,0x00,0x09,0x01,0x00,0x09,0x02,0x01,0x09,0x01,0x11,0x09,0x02,0x02,0x08},
{0x26,0x10,0x1B,0x00,0x04,0x09,0x02,0x03,0x08,0x33,0x09,0x02,0x04,0x28,0x00,0x09,0x02,0x05,0x28,0x01,0x09,0x02,0x06,0x35,0xDF,0x35,0xDD,0x08,0x22,0x25,0xD9},
{0x26,0x10,0x1B,0x00,0x05,0x01,0x09,0x05,0xA1,0x01,0x85,0x10,0x15,0x00,0x26,0xFF,0x00,0x75,0x08,0x95,0x01,0x06,0x00,0xFF,0x09,0x01,0x91,0x00,0x85,0x11,0x95},
{0x26,0x10,0x1B,0x00,0x01,0x09,0x01,0x91,0x00,0x85,0x12,0x95,0x02,0x09,0x01,0x91,0x00,0x85,0x13,0x95,0x01,0x09,0x01,0x91,0x00,0x85,0x14,0x95,0x01,0x09,0x01},
{0x26,0x10,0x18,0x00,0x91,0x00,0x85,0x15,0x95,0x01,0x09,0x01,0x91,0x00,0x85,0x16,0x95,0x15,0x09,0x01,0x91,0x00,0x85,0x17,0x95,0x02,0x01,0x62,0xFF,0xFF,0xFF}
};

unsigned char att4d[5][31]=
{
{0x26,0x20,0x1B,0x00,0x80,0x00,0x40,0x00,0x07,0x00,0x03,0x00,0x7B,0x00,0x76,0x06,0x09,0x01,0x91,0x00,0x85,0x18,0x95,0x15,0x09,0x01,0x91,0x00,0x85,0x19,0x95},
{0x26,0x10,0x1B,0x00,0x01,0x09,0x01,0x91,0x00,0x85,0x1A,0x95,0x01,0x09,0x01,0x91,0x00,0x85,0x20,0x95,0x06,0x09,0x01,0x81,0x00,0x85,0x21,0x95,0x15,0x09,0x01},
{0x26,0x10,0x1B,0x00,0x81,0x00,0x85,0x22,0x95,0x04,0x09,0x01,0x81,0x00,0x85,0x30,0x95,0x02,0x09,0x01,0x81,0x00,0x85,0x31,0x95,0x05,0x09,0x01,0x81,0x00,0x85},
{0x26,0x10,0x1B,0x00,0x32,0x95,0x0A,0x09,0x01,0x81,0x00,0x85,0x33,0x95,0x11,0x09,0x01,0x81,0x00,0x85,0x34,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x35,0x95,0x15},
{0x26,0x10,0x18,0x00,0x09,0x01,0x81,0x00,0x85,0x36,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x37,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x02,0x01,0xD8,0xFF,0xFF,0xFF}
};

unsigned char att4e[5][31]=
{
{0x26,0x20,0x1B,0x00,0x80,0x00,0x40,0x00,0x07,0x00,0x04,0x00,0x7B,0x00,0x76,0x3D,0x95,0x15,0x09,0x01,0x81,0x00,0x85,0x3E,0x95,0x15,0x09,0x01,0x81,0x00,0x85},
{0x26,0x10,0x1B,0x00,0x3F,0x95,0x15,0x09,0x01,0x81,0x00,0xC0,0x09,0x02,0x07,0x35,0x08,0x35,0x06,0x09,0x04,0x09,0x09,0x01,0x00,0x09,0x02,0x08,0x28,0x00,0x09},
{0x26,0x10,0x1B,0x00,0x02,0x09,0x28,0x01,0x09,0x02,0x0A,0x28,0x01,0x09,0x02,0x0B,0x09,0x01,0x00,0x09,0x02,0x0C,0x09,0x0C,0x80,0x09,0x02,0x0D,0x28,0x00,0x09},
{0x26,0x10,0x1B,0x00,0x02,0x0E,0x28,0x00,0x36,0x00,0x5A,0x09,0x00,0x00,0x0A,0x00,0x01,0x00,0x01,0x09,0x00,0x01,0x35,0x03,0x19,0x12,0x00,0x09,0x00,0x04,0x35},
{0x26,0x10,0x18,0x00,0x0D,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x01,0x35,0x03,0x19,0x00,0x01,0x09,0x00,0x05,0x35,0x03,0x19,0x10,0x02,0x02,0x4E,0xFF,0xFF,0xFF}
};

unsigned char att4f[3][31]=
{
{0x26,0x20,0x1B,0x00,0x39,0x00,0x40,0x00,0x07,0x00,0x05,0x00,0x34,0x00,0x31,0x02,0x09,0x00,0x09,0x35,0x08,0x35,0x06,0x19,0x12,0x00,0x09,0x01,0x00,0x09,0x02},
{0x26,0x10,0x1B,0x00,0x00,0x09,0x01,0x00,0x09,0x02,0x01,0x09,0x05,0x7E,0x09,0x02,0x02,0x09,0x03,0x06,0x09,0x02,0x03,0x09,0x06,0x00,0x09,0x02,0x04,0x28,0x01},
{0x26,0x10,0x07,0x00,0x09,0x02,0x05,0x09,0x00,0x02,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF}
};

unsigned char att5[4][31]=
{
{0x26,0x20,0x1B,0x00,0x68,0x00,0x40,0x00,0x07,0x00,0x06,0x00,0x63,0x00,0x60,0x36,0x00,0x5D,0x36,0x00,0x5A,0x09,0x00,0x00,0x0A,0x00,0x01,0x00,0x01,0x09,0x00},
{0x26,0x10,0x1B,0x00,0x01,0x35,0x03,0x19,0x12,0x00,0x09,0x00,0x04,0x35,0x0D,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x01,0x35,0x03,0x19,0x00,0x01,0x09,0x00,0x05},
{0x26,0x10,0x1B,0x00,0x35,0x03,0x19,0x10,0x02,0x09,0x00,0x09,0x35,0x08,0x35,0x06,0x19,0x12,0x00,0x09,0x01,0x00,0x09,0x02,0x00,0x09,0x01,0x00,0x09,0x02,0x01},
{0x26,0x10,0x1B,0x00,0x09,0x05,0x7E,0x09,0x02,0x02,0x09,0x03,0x06,0x09,0x02,0x03,0x09,0x06,0x00,0x09,0x02,0x04,0x28,0x01,0x09,0x02,0x05,0x09,0x00,0x02,0x00}
};

//******************************************************************************
//******************************************************************************
// Local Routines
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        InitializeSystem
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if successful, FALSE if not.
 *
 * Side Effects:    See below
 *
 * Overview:        This routine initializes the processor and peripheral,
 *                  setting clock speeds and enabling any required
 *                  features.
 *************************************************************************/

BOOL InitializeSystem ( void )
{

	unsigned int pll_startup_counter = 600;
	CLKDIVbits.PLLEN = 1;
	while(pll_startup_counter--);

	// Configure U2RX - put on pin 19 (RP8)
	RPINR19bits.U2RXR = 8;
	// Configure U2TX - put on pin 5  (RP7)
	RPOR3bits.RP7R = 5;
    // Init UART
    UART2Init();

    // Set Default demo state
    DemoState = BT_INITIALIZE;
	HciState=HCI_CMD_RESET;
    return TRUE;
} // InitializeSystem


/*************************************************************************
 * Function:        CheckForNewAttach
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          deviceAddress (global)
 *                  Updates the device address when an attach is found.
 *
 * Returns:         TRUE if a new device has been attached.  FALSE,
 *                  otherwise.
 *
 * Side Effects:    Prints attach message
 *
 * Overview:        This routine checks to see if a new device has been
 *                  attached.  If it has, it records the address.
 *************************************************************************/

BOOL CheckForNewAttach ( void )
{
    // Try to get the device address, if we don't have one.
    if (deviceAddress == 0)
    {
        GENERIC_DEVICE_ID DevID;

        DevID.vid   = 0x0A12;
        DevID.pid   = 0x0001;
        #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
            DevID.serialNumberLength = 0;
            DevID.serialNumber = NULL;
        #endif

        if (USBHostGenericGetDeviceAddress(&DevID))
        {
            deviceAddress = DevID.deviceAddress;
			#ifdef DEBUG_MODE
            UART2PrintString( "Generic demo device attached - polled, deviceAddress=" );
            UART2PutDec( deviceAddress );
            UART2PrintString( "\r\n" );
			#endif
            return TRUE;
        }
    }

    return FALSE;

} // CheckForNewAttach

/*************************************************************************
 * Function:        ManageDemoState
 *
 * Preconditions:   The DemoState global variable must be initialized to
 *                  BT_STATE_IDLE (0).  (This occurs on reset.)
 *
 * Input:           DemoState (global)
 *                  Actions selected based value of DemoState on function
 *                  entry.
 *
 *                  deviceAddress (global)
 *                  May use device address to access device, depending on
 *                  state.
 *
 *                  DataPacket (global)
 *                  May read data from packet buffer, depending on state.
 *
 * Output:          DemoState (global)
 *                  Updates demo state as appropriate.
 *
 *                  DataPacket (global)
 *                  May cause data in the packet buffer to be updated,
 *                  depending on state.
 *
 * Returns:         None
 *
 * Side Effects:    Depend on state transition
 *
 * Overview:        This routine maintains the state of the application,
 *                  updateing global data and taking actions as necessary
 *                  to maintain the custom demo operations.
 *************************************************************************/
void ManageDemoState ( void )
{
    BYTE RetVal;

    // Watch for device detaching
    if (USBHostGenericDeviceDetached(deviceAddress) && deviceAddress != 0)
    {
		#ifdef DEBUG_MODE
        UART2PrintString( "Generic demo device detached - polled\r\n" );
		#endif
        DemoState = BT_INITIALIZE;
		HciState=HCI_CMD_RESET;
        deviceAddress   = 0;
    }

    switch (DemoState)
    {
    case BT_INITIALIZE:
        DemoState = BT_STATE_IDLE;
        break;

    /** Idle State:  Loops here until attach **/
    case BT_STATE_IDLE:
        if (CheckForNewAttach())
        {
			DemoState = BT_STATE_PROCESS;
			HciState = HCI_CMD_RESET;
        }
        break;

    case BT_STATE_PROCESS:
		switch (HciState)
		{
//HCI layer***********************************************************************
		case HCI_CMD_RESET:
			buf1[0]=0x03;
			buf1[1]=0x0c;
			buf1[2]=0;
			data_size=3;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_RESET_WRITE_END;
		    break;

		case HCI_CMD_RESET_WRITE_END:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_RESET_READ_END;
		    break;

		case HCI_CMD_RESET_READ_END:

			if(buf1[0]==0xff) {HciState = HCI_CMD_READ_BD_ADDR;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "HCI_CMD_RESET " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState =HCI_CMD_RESET_WRITE_END;
			break;
//********************************************************************************
		case HCI_CMD_READ_BD_ADDR:
			buf1[0]=0x09;
			buf1[1]=0x10;
			buf1[2]=0;
			data_size=3;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_READ_BD_ADDR_WRITE_END;
			break;

		case HCI_CMD_READ_BD_ADDR_WRITE_END:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_READ_BD_ADDR_READ_END;
		    break;

		case HCI_CMD_READ_BD_ADDR_READ_END:
			if(buf1[0]==0xff) {HciState = HCI_CMD_LOCAL_NAME;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "HCI_READ_BD_ADDR " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState =HCI_CMD_READ_BD_ADDR_WRITE_END;
		    break;
//********************************************************************************
		case HCI_CMD_LOCAL_NAME:
			buf1[0]=0x13;
			buf1[1]=0x0c;
			buf1[2]=0x04;
			buf1[3]='y';
			buf1[4]='t';
			buf1[5]='s';
			buf1[6]=0x00;
			data_size=7;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_LOCAL_NAME_WRITE_END;
			break;

		case HCI_CMD_LOCAL_NAME_WRITE_END:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_LOCAL_NAME_READ_END;
		    break;

		case HCI_CMD_LOCAL_NAME_READ_END:
			if(buf1[0]==0xff) {HciState = HCI_CMD_CLASS_DEVICE;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "HCI_CMD_LOCAL_NAME  " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = HCI_CMD_LOCAL_NAME_WRITE_END;
		    break;
//********************************************************************************
		case HCI_CMD_CLASS_DEVICE:
			buf1[0]=0x24;
			buf1[1]=0x0c;
			buf1[2]=0x03;
			buf1[3]=0x04;//joystick
			buf1[4]=0x05;//joystick
			buf1[5]=0x00;

			data_size=6;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_CLASS_DEVICE_WRITE_END;
			break;

		case HCI_CMD_CLASS_DEVICE_WRITE_END:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_CLASS_DEVICE_READ_END;
		    break;

		case HCI_CMD_CLASS_DEVICE_READ_END:

			if(buf1[0]==0xff) {HciState = HCI_CMD_SCAN_ENABLE;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "HCI_CMD_CLASS_DEVICE " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = HCI_CMD_CLASS_DEVICE_WRITE_END;
		    break;
//********************************************************************************
		case HCI_CMD_SCAN_ENABLE:
			buf1[0]=0x1a;
			buf1[1]=0x0c;
			buf1[2]=0x01;
			buf1[3]=0x03;
			data_size=4;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_SCAN_ENABLE_WRITE_END;
			break;

		case HCI_CMD_SCAN_ENABLE_WRITE_END:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_SCAN_ENABLE_READ_END;
		    break;

		case HCI_CMD_SCAN_ENABLE_READ_END:

			if(buf1[0]==0xff) {HciState = HCI_CMD_WAIT_CONNECTION;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "HCI_CMD_SCAN_ENABLE " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = HCI_CMD_SCAN_ENABLE_WRITE_END;
		    break;
//********************************************************************************
		case HCI_CMD_WAIT_CONNECTION:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_WAIT_CONNECTION_READ_END;
			break;

		case HCI_CMD_WAIT_CONNECTION_READ_END:
			if(buf1[0]!=0x04) {HciState = HCI_CMD_WAIT_CONNECTION;break;}
			//if(buf1[0]==0xff) {HciState = HCI_CMD_CONNECTION_ACCEPT;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "HCI_CMD_WAIT_CONNECTION " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = HCI_CMD_CONNECTION_ACCEPT;

		    break;
//********************************************************************************
		case HCI_CMD_CONNECTION_ACCEPT:

			buf1[8]=buf1[7];//copy address
			buf1[7]=buf1[6];
			buf1[6]=buf1[5];
			buf1[5]=buf1[4];
			buf1[4]=buf1[3];
			buf1[3]=buf1[2];
			buf1[0]=0x09;
			buf1[1]=0x04;
			buf1[2]=0x07;
			buf1[9]=0x01;//role
			data_size=10;
			DemoState = BT_STATE_WRITE_CLASS;
			HciState = HCI_CMD_CONNECTION_ACCEPT_WRITE_END;
			break;

		case HCI_CMD_CONNECTION_ACCEPT_WRITE_END:
			DemoState = BT_STATE_READ_EP1;
			HciState = HCI_CMD_CONNECTION_ACCEPT_READ_END;
		    break;

		case HCI_CMD_CONNECTION_ACCEPT_READ_END:
//			if(buf1[0]!=0x03){ DemoState =BT_STATE_READ_EP1; break;}
			handle[0]=buf1[3];handle[1]=buf1[4]+0x20;

			#ifdef DEBUG_MODE
			UART2PrintString( "HCI_CMD_CONNECTION_ACCEPT " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			hid_flag=0;//first stage for ACL connection
			//there are four stages for ACL connection in case of bluetooth HID

			HciState = L2CAP_CON_REQ;
			HciState = OUT1;
		    break;

//********************************************************************************
		case  OUT1:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END1;
			break;

		case OUT_END1:
			if(buf1[0]==0x03) {handle[0]=buf1[3];handle[1]=buf1[4]+0x20;}
			if(buf1[0]==0xff) {HciState = L2CAP_CON_REQ;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "OUT1: " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT1;
		    break;

//L2CAP layer*********************************************************************

		case  L2CAP_CON_REQ:
			delay_time=0;
			DemoState = BT_STATE_READ_ACL;
			HciState = L2CAP_CON_REQ_READ_END;
			break;

		case L2CAP_CON_REQ_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "L2CAP_CON_REQ \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
			if(buf[12]==0x11) hid_flag=2;
			HciState = L2CAP_CON_RESP;
		    break;
//********************************************************************************
		case  L2CAP_CON_RESP:
			buf[2]=0x10;
			buf[4]=0x0c;
			buf[8]=0x03;//
			buf[10]=0x08;
			buf[12]=buf[14]+4;//dst_cid (PIC)
			buf[13]=buf[15];//
			buf[16]=0x00;//result
			buf[17]=0x00;//
			buf[18]=0x00;//statis
			buf[19]=0x00;//
			data_size=20;
			src_cid[0]=buf[14];
			src_cid[1]=buf[15];		
			dst_cid[0]=buf[12];
			dst_cid[1]=buf[13];

			if(hid_flag==2){
				src_cid1[0]=buf[14];
				src_cid1[1]=buf[15];		
				dst_cid1[0]=buf[12];
				dst_cid1[1]=buf[13];
			}			

			DemoState = BT_STATE_WRITE_ACL;
//			HciState = L2CAP_CONFIG_REQ;
			HciState = OUT2;
			break;
//********************************************************************************
		case  OUT2:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END2;
			break;

		case OUT_END2:
			if(buf1[0]==0xff) {HciState = L2CAP_CONFIG_REQ;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "OUT2:" );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT2;
		    break;
//********************************************************************************
		case  L2CAP_CONFIG_REQ:

			DemoState = BT_STATE_READ_ACL;
			HciState = L2CAP_CONFIG_REQ_READ_END;
			break;

		case L2CAP_CONFIG_REQ_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "L2CAP_CONFIG_REQ \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = L2CAP_CONFIG_RESP;
		    break;
//********************************************************************************
		case  L2CAP_CONFIG_RESP:
			buf[18]=0x01;//buf[16];//option
			buf[19]=0x02;//buf[17];//
			buf[20]=0x01;//0x00;//buf[18];//
			buf[21]=0x00;//0x01;//buf[19];//
			buf[2]=0x12;
			buf[4]=0x0e;
			buf[8]=0x05;//
			buf[10]=0x0a;
			buf[12]=src_cid[0];
			buf[13]=src_cid[1];
			buf[16]=0x00;//result
			buf[17]=0x00;//
			data_size=22;
			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT3;
			break;

//********************************************************************************
		case  OUT3:

			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END3;
			break;

		case OUT_END3:
			if(buf1[0]==0xff) {HciState =L2CAP_CONFIG_REQ_HOST;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "OUT3:" );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT3;
		    break;
//********************************************************************************
		case  L2CAP_CONFIG_REQ_HOST:
			buf[16]=0x01;//buf[18];//option
			buf[17]=0x02;//buf[19];//
			buf[18]=0x00;//buf[20];//
			buf[19]=0x01;//buf[21];//
			buf[2]=0x10;
			buf[4]=0x0c;
			buf[8]=0x04;//
			buf[10]=0x08;
			buf[12]=src_cid[0];
			buf[13]=src_cid[1];
			buf[14]=0x00;//result
			buf[15]=0x00;//
			data_size=20;
			DemoState = BT_STATE_WRITE_ACL;
			HciState =OUT4;
			break;
//********************************************************************************
		case  OUT4:

			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END4;
			break;

		case OUT_END4:
			if(buf1[0]==0xff) {HciState =L2CAP_CONFIG_RESP_HOST;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "OUT4: " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT4;
		    break;

//********************************************************************************

		case  L2CAP_CONFIG_RESP_HOST:
			DemoState = BT_STATE_READ_ACL;
			HciState = L2CAP_CONFIG_REQ_HOST_READ_END;
			break;

		case L2CAP_CONFIG_REQ_HOST_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "L2CAP_CONFIG_REQ_HOST \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			
			if(hid_flag==3) HciState = HID_START;    //connection complete
			else if(hid_flag==2) {HciState = L2CAP_CON_REQ; hid_flag=3;} //GO TO final stage for L2CAP connection
			else HciState = SDP_SEARCH_REQ;//GO TO first or second stage for L2CAP connection
		    break;

//SDP layer **********************************************************************
		case  SDP_SEARCH_REQ:
			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_SEARCH_REQ_READ_END;
			break;

		case SDP_SEARCH_REQ_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "SDP_SEARCH_REQ \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = SDP_SERCH_RESP;
			if(buf[8]==0x06) HciState = SDP_SERCH_RESP_MICRO;//for Microsoft 
		    break;
//
//
//********************************************************************************
		case  SDP_SERCH_RESP:

			buf[2]=0x12;
			buf[4]=0x0e;

			buf[6]=src_cid[0];
			buf[7]=src_cid[1];
			buf[8]=0x03;

			buf[12]=0x09;
			buf[13]=0x00;//total service record
			buf[14]=0x01;
			buf[15]=0x00;//current service record
			buf[16]=0x01;
			buf[17]=0x00;//serevice record hadle list
			buf[18]=0x01;
			buf[19]=0x00;
			buf[20]=0x00;
			buf[21]=0x00;//continuation state

			data_size=22;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT5;
			break;


//********************************************************************************
		case  OUT5:

			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END5;
			break;

		case OUT_END5:
			if(buf1[0]==0xff) {HciState =SDP_ATTR_REQ;break;}
			#ifdef DEBUG_MODE
			UART2PrintString( "OUT5: " );
			for(data_num=0;data_num<buf1[1]+2;data_num++)
      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT5;
		    break;

//********************************************************************************

		case  SDP_ATTR_REQ:
			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_REQ_READ_END;
			break;

		case SDP_ATTR_REQ_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "SDP_ATTR_REQ \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

//Initialization for SDP attribute data
//copy CID to SDP data
			for(step=0;step<5;step++){
			 	att2a[step][0]=buf[0];
				att2b[step][0]=buf[0];
				att2c[step][0]=buf[0];
				att2d[step][0]=buf[0];
			}
//copy src_cid to SDP data		
			att2a[0][6]=src_cid[0];	att2a[0][7]=src_cid[1];
			att2b[0][6]=src_cid[0];	att2b[0][7]=src_cid[1];
			att2c[0][6]=src_cid[0];	att2c[0][7]=src_cid[1];
			att2d[0][6]=src_cid[0];	att2d[0][7]=src_cid[1];

//copy ID to SDP data
			att2a[0][9]=buf[9];att2a[0][10]=buf[10];

			step=0;
			HciState =SDP_ATTR_RESP2a;

		    break;

//********************************************************************************
		case  SDP_ATTR_RESP2a:

			if(step==5) {HciState = SDP_ATTR_RESP2a_READ;step=0;break;}

			data_size=att2a[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) buf[pdata]=att2a[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
//			for(data_num=0;data_num<data_size;data_num++)
//     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT7;
			break;
//********************************************************************************
		case  OUT7:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END7;
			break;

		case OUT_END7:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP2a;break;}
			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT7:" );
////			for(data_num=0;data_num<buf1[1]+2;data_num++)
 //     	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT7;
		    break;

//********************************************************************************
		case  SDP_ATTR_RESP2a_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP2a_READ_END;
			break;

		case SDP_ATTR_RESP2a_READ_END:

			#ifdef DEBUG_MODE
///			UART2PrintString( "2a_SDP_ATTR_RESP \r\n" );
//			for(data_num=0;data_num<buf[2]+4;data_num++)
//      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att2b[0][9]=buf[9];att2b[0][10]=buf[10];
			HciState = SDP_ATTR_RESP2b;
		    break;

//********************************************************************************
		case  SDP_ATTR_RESP2b:

			if(step==5) {HciState = SDP_ATTR_RESP2b_READ;step=0;break;}
	
			data_size=att2b[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att2b[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
//			for(data_num=0;data_num<data_size;data_num++)
 //     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT8;
			break;
//********************************************************************************
		case  OUT8:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END8;
			break;

		case OUT_END8:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP2b;break;}
			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT8:" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
//      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT8;
		    break;

//********************************************************************************
		case  SDP_ATTR_RESP2b_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP2b_READ_END;
			break;

		case SDP_ATTR_RESP2b_READ_END:

			#ifdef DEBUG_MODE
//			UART2PrintString( "2b_SDP_ATTR_RESP \r\n" );
//			for(data_num=0;data_num<buf[2]+4;data_num++)
//      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att2c[0][9]=buf[9];att2c[0][10]=buf[10];

			HciState =SDP_ATTR_RESP2c;
//			HciState = SDP_ATTR_RESP3;
		    break;

//********************************************************************************
		case  SDP_ATTR_RESP2c:

			if(step==5) {HciState = SDP_ATTR_RESP2c_READ;step=0;break;}
	
			data_size=att2c[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att2c[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
//			for(data_num=0;data_num<data_size;data_num++)
 //     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT9;
			break;

//********************************************************************************
		case  OUT9:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END9;
			break;

		case OUT_END9:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP2c;break;}

			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT9 :" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
 //     	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT9;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP2c_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP2c_READ_END;
			break;

		case SDP_ATTR_RESP2c_READ_END:

			#ifdef DEBUG_MODE
//			UART2PrintString( "2c_SDP_ATTR_RESP \r\n" );
//			for(data_num=0;data_num<buf[2]+4;data_num++)
//      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att2d[0][9]=buf[9];att2d[0][10]=buf[10];

			HciState =SDP_ATTR_RESP2d;
		    break;

//********************************************************************************

		case  SDP_ATTR_RESP2d:

		if(step==5) {HciState = SDP_ATTR_RESP2d_READ;step=0;break;}
	
			data_size=att2d[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att2d[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
//			for(data_num=0;data_num<data_size;data_num++)
//      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT10;
			break;
//********************************************************************************
		case  OUT10:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END10;
			break;

		case OUT_END10:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP2d;break;}

			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT10 :" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
//      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT10;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP2d_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP2d_READ_END;
			break;

		case SDP_ATTR_RESP2d_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "2d_SDP_ATTR_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			HciState =SDP_ATTR_END;
		    break;
//********************************************************************************
		case SDP_SERCH_RESP_MICRO:

//Initialization for SDP attribute data
//copy CID to SDP data
			for(step=0;step<4;step++){
			 	att5[step][0]=buf[0];
			}
			for(step=0;step<5;step++){
			 	att4a[step][0]=buf[0];
				att4b[step][0]=buf[0];
				att4c[step][0]=buf[0];
				att4d[step][0]=buf[0];
			}
//copy src_cid to SDP data		
			att4a[0][6]=src_cid[0];	att4a[0][7]=src_cid[1];
			att4b[0][6]=src_cid[0];	att4b[0][7]=src_cid[1];
			att4c[0][6]=src_cid[0];	att4c[0][7]=src_cid[1];
			att4d[0][6]=src_cid[0];	att4d[0][7]=src_cid[1];
			att5[0][6]=src_cid[0];	att5[0][7]=src_cid[1];
//copy ID to SDP data
			att4a[0][9]=buf[9];att4a[0][10]=buf[10];

			step=0;

			HciState =SDP_ATTR_RESP4a;
		    break;

//********************************************************************************
		case  SDP_ATTR_RESP4a:

			if(step==5) {HciState = SDP_ATTR_RESP4a_READ;step=0;break;}

			data_size=att4a[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) buf[pdata]=att4a[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
			for(data_num=0;data_num<data_size;data_num++)
     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT7a;
			break;

//********************************************************************************
		case  OUT7a:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END7a;
			break;

		case OUT_END7a:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP4a;break;}
			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT7a:" );
////			for(data_num=0;data_num<buf1[1]+2;data_num++)
 //     	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT7a;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP4a_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP4a_READ_END;
			break;

		case SDP_ATTR_RESP4a_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "4a_SDP_ATTR_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att4b[0][9]=buf[9];att4b[0][10]=buf[10];
			HciState = SDP_ATTR_RESP4b;
		    break;

//********************************************************************************
		case  SDP_ATTR_RESP4b:

			if(step==5) {HciState = SDP_ATTR_RESP4b_READ;step=0;break;}
	
			data_size=att4b[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att4b[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
			for(data_num=0;data_num<data_size;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT8a;
			break;
//********************************************************************************
		case  OUT8a:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END8a;
			break;

		case OUT_END8a:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP4b;break;}
			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT8:" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
//      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT8a;
		    break;

//********************************************************************************
		case  SDP_ATTR_RESP4b_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP4b_READ_END;
			break;

		case SDP_ATTR_RESP4b_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "4b_SDP_ATTR_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att4c[0][9]=buf[9];att4c[0][10]=buf[10];

			HciState =SDP_ATTR_RESP4c;
//			HciState = SDP_ATTR_RESP3;
		    break;

//********************************************************************************
		case  SDP_ATTR_RESP4c:

			if(step==5) {HciState = SDP_ATTR_RESP4c_READ;step=0;break;}
	
			data_size=att4c[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att4c[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
			for(data_num=0;data_num<data_size;data_num++)
     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT9a;
			break;

//********************************************************************************
		case  OUT9a:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END9a;
			break;

		case OUT_END9a:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP4c;break;}

			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT9a :" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
 //     	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT9a;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP4c_READ:
			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP4c_READ_END;
			break;

		case SDP_ATTR_RESP4c_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "4c_SDP_ATTR_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att4d[0][9]=buf[9];att4d[0][10]=buf[10];

			HciState =SDP_ATTR_RESP4d;
		    break;

//********************************************************************************

		case  SDP_ATTR_RESP4d:

		if(step==5) {HciState = SDP_ATTR_RESP4d_READ;step=0;break;}
	
			data_size=att4d[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att4d[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
			for(data_num=0;data_num<data_size;data_num++)
     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT10a;
			break;
//********************************************************************************
		case  OUT10a:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END10a;
			break;

		case OUT_END10a:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP4d;break;}

			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT10 :" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
//      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT10a;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP4d_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP4d_READ_END;
			break;

		case SDP_ATTR_RESP4d_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "4d_SDP_ATTR_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att4e[0][9]=buf[9];att4e[0][10]=buf[10];

			HciState =SDP_ATTR_RESP4e;
//			if(hid_flag==1) HciState =SDP_ATTR_END;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP4e:

		if(step==5) {HciState = SDP_ATTR_RESP4e_READ;step=0;break;}
	
			data_size=att4e[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att4e[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
			for(data_num=0;data_num<data_size;data_num++)
     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT11a;
			break;
//********************************************************************************
		case  OUT11a:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END11a;
			break;

		case OUT_END11a:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP4e;break;}

			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT11a :" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
//      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT11a;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP4e_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP4e_READ_END;
			break;

		case SDP_ATTR_RESP4e_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "4e_SDP_ATTR_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att4f[0][9]=buf[9];att4f[0][10]=buf[10];

			HciState =SDP_ATTR_RESP4f;
//			if(hid_flag==1) HciState =SDP_ATTR_END;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP4f:

		if(step==3) {HciState = SDP_ATTR_RESP4f_READ;step=0;break;}
	
			data_size=att4f[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att4f[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
			for(data_num=0;data_num<data_size;data_num++)
     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT12a;
			break;
//********************************************************************************
		case  OUT12a:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END12a;
			break;

		case OUT_END12a:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP4f;break;}

			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT12a :" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
//      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT12a;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP4f_READ:

			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP4f_READ_END;
			break;

		case SDP_ATTR_RESP4f_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "5_SDP_ATTR_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att5[0][9]=buf[9];att5[0][10]=buf[10];

			HciState =SDP_ATTR_RESP5;

		    break;
//********************************************************************************
		case  SDP_ATTR_RESP5:

		if(step==4) {HciState = SDP_ATTR_RESP5_READ;step=0;break;}
	
			data_size=att5[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att5[step][pdata];
			step++; 

			#ifdef DEBUG_MODE
			for(data_num=0;data_num<data_size;data_num++)
     	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = OUT13a;
			break;
//********************************************************************************
		case  OUT13a:
			buf1[0]=0xff;
			DemoState = BT_STATE_READ_EP1;
			HciState = OUT_END13a;
			break;

		case OUT_END13a:
			if(buf1[0]==0xff) { HciState = SDP_ATTR_RESP5;break;}

			#ifdef DEBUG_MODE
//			UART2PrintString( "OUT13a :" );
//			for(data_num=0;data_num<buf1[1]+2;data_num++)
//      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
//			UART2PrintString( "\r\n" );
			#endif

			HciState = OUT13a;
		    break;
//********************************************************************************
		case  SDP_ATTR_RESP5_READ:
			DemoState = BT_STATE_READ_ACL;
			HciState = SDP_ATTR_RESP5_READ_END;
			break;

		case SDP_ATTR_RESP5_READ_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "5_SDP_ATTR_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif
//copy ID to SDP data
			att5[0][9]=buf[9];att5[0][10]=buf[10];

			HciState =SDP_ATTR_END;
			hid_flag=1;//GO TO third stage
		    break;


//********************************************************************************
		case  SDP_ATTR_END:

			buf[8]=0x07;
			data_size=16;
	
//			DemoState = BT_STATE_WRITE_ACL;
			HciState = L2CAP_CON_REQ;

			hid_flag++;//GO TO second or third stage for L2CAP connection

			break;
//HID*****************************************************************************
		case  HID_START:

			#ifdef DEBUG_MODE
			UART2PrintString( "HID START\r\n" );
			#endif

			DemoState = BT_STATE_READ_ACL;
			HciState = HID_START_RESP;
			break;

		case  HID_START_RESP:

//			#ifdef DEBUG_MODE
			UART2PrintString( "HID_START_RESP \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
//			#endif

			buf[2]=0x08;
			buf[3]=0x00;
			buf[4]=0x04;
			buf[5]=0x00;
			buf[6]=src_cid[0];//0x41;
			buf[7]=src_cid[1];//0x00;
			buf[8]=0xa1;
			buf[9]=0x30;
			buf[10]=0x00;
			buf[11]=0x00;
			data_size=12;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = HID_HANDSHAKE_ERR;
		    break;

		case  HID_HANDSHAKE_ERR:
			buf[2]=0x05;
			buf[3]=0x00;
			buf[4]=0x01;
			buf[5]=0x00;
			buf[6]=src_cid1[0];//0x40;
			buf[7]=src_cid1[1];//0x00;
			buf[8]=0x03;
			data_size=9;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = HID_READ_DATA;
		    break;

		case  HID_READ_DATA://read
			DemoState = BT_STATE_READ_ACL;
			HciState = HID_READ_DATA_END;
			break;

		case HID_READ_DATA_END:

			#ifdef DEBUG_MODE
			UART2PrintString( "HID_READ_DATA_END \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			for(step=10;step<31;step++) buf1[step]=buf[step];

			HciState =HID_HANDSHAKE_SUCCESS;
		    break;

		case  HID_HANDSHAKE_SUCCESS:
			buf[2]=0x05;
			buf[3]=0x00;
			buf[4]=0x01;
			buf[5]=0x00;
			buf[6]=src_cid1[0];//0x40;
			buf[7]=src_cid1[1];//0x00;
			buf[8]=0x00;
			data_size=9;

			DemoState = BT_STATE_WRITE_ACL;
			HciState = HID_WRITE_DATA;
		    break;

		case  HID_WRITE_DATA://write

			for(step=10;step<31;step++) buf[step]=buf1[step];
			buf[0]=handle[0];
			buf[1]=handle[1];
			buf[2]=0x1b;
			buf[3]=0x00;
			buf[4]=0x17;
			buf[5]=0x00;
			buf[6]=src_cid[0];//0x41;
			buf[7]=src_cid[1];//0x00;
			buf[8]=0xa1;
			buf[9]=0x21;

			buf[10]+=1;
			data_size=31;

			#ifdef DEBUG_MODE
			UART2PrintString( "HID_WRITE_DATA \r\n" );
			for(data_num=0;data_num<buf[2]+4;data_num++)
      	          {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			DemoState = BT_STATE_WRITE_ACL;
			HciState = HID_READ_DATA;
		    break;

		case  PROG_END:
			break;

		}
		break;

//********************************************************************************
//@READ-WRITE FUNCTIONS
//********************************************************************************
//WRITE ENDPOINT 0
    case BT_STATE_WRITE_CLASS:
        if (!USBHostGenericTxIsBusy(deviceAddress))
		{
            if ( (RetVal=USBHostGenericClassRequest( deviceAddress, buf1, data_size )) == USB_SUCCESS )
            {
			//UART2PrintString( "HCI COMMAND SENT\r\n" );	
            DemoState = BT_STATE_PROCESS;
            }
            else
            {
			UART2PrintString( "Write Class Error !\r\n" );	
            }
        }
        break;

//READ ENDPOINT 1
    case BT_STATE_READ_EP1:
        if (!USBHostGenericRxIsBusy(deviceAddress))
        {
            if ( (RetVal=USBHostGenericRead(deviceAddress, buf1, DATA_PACKET_LENGTH)) == USB_SUCCESS )
            {
                DemoState = BT_STATE_READ_CLASS_WAITING;
                //UART2PrintString( "READ EP1\r\n" );
            }
            else
            {
                UART2PrintString( "Device Read Error !\r\n" );
            }
        }
        break;

    case BT_STATE_READ_CLASS_WAITING:
        if (!USBHostGenericRxIsBusy(deviceAddress))
           DemoState = BT_STATE_PROCESS;
        break;


//WRITE ENDPOINT 2
    case BT_STATE_WRITE_ACL:
        if (!USBHostGenericTxIsBusy(deviceAddress))
		{
            if ( (RetVal=USBHostGenericAclWrite( deviceAddress, buf, data_size )) == USB_SUCCESS )
            {
			//UART2PrintString( "HCI COMMAND SENT\r\n" );	
            DemoState = BT_STATE_PROCESS;
            }
            else
            {
			UART2PrintString( "Write Acl Error !\r\n" );	
            }
        }
        break;

//READ ENDPOINT 2
    case BT_STATE_READ_ACL:
        if (!USBHostGenericRxIsBusy(deviceAddress))
        {
            if ( (RetVal=USBHostGenericAclRead(deviceAddress, buf, DATA_PACKET_LENGTH)) == USB_SUCCESS )
            {
                DemoState = BT_STATE_READ_ACL_WAITING;
//				#ifdef DEBUG_MODE
//                UART2PrintString( "READ EP2\r\n" );
//				#endif
            }
            else
            {
                UART2PrintString( "Read Acl Error !\r\n" );
            }
        }
        break;

    case BT_STATE_READ_ACL_WAITING:
        if (!USBHostGenericRxIsBusy(deviceAddress))
           DemoState = BT_STATE_PROCESS;
        break;


    /** Error state:  Hold here until detached **/
    case BT_STATE_ERROR:                          // To Do: Flash LEDs
        break;

    default:
        DemoState = BT_INITIALIZE;
        break;
    }

    DelayMs(1); // 1ms delay

} // ManageDemoState


//******************************************************************************
//******************************************************************************
// USB Support Functions
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        USB_ApplicationEventHandler
 *
 * Preconditions:   The USB must be initialized.
 *
 * Input:           event       Identifies the bus event that occured
 *
 *                  data        Pointer to event-specific data
 *
 *                  size        Size of the event-specific data
 *
 * Output:          deviceAddress (global)
 *                  Updates device address when an attach or detach occurs.
 *
 *                  DemoState (global)
 *                  Updates the demo state as appropriate when events occur.
 *
 * Returns:         TRUE if the event was handled, FALSE if not
 *
 * Side Effects:    Event-specific actions have been taken.
 *
 * Overview:        This routine is called by the Host layer or client
 *                  driver to notify the application of events that occur.
 *                  If the event is recognized, it is handled and the
 *                  routine returns TRUE.  Otherwise, it is ignored (or
 *                  just "sniffed" and the routine returns FALSE.
 *************************************************************************/

BOOL USB_ApplicationEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
        BYTE i;
    #endif

    // Handle specific events.
    switch (event)
    {
        case EVENT_GENERIC_ATTACH:
            if (size == sizeof(GENERIC_DEVICE_ID))
            {
                deviceAddress   = ((GENERIC_DEVICE_ID *)data)->deviceAddress;
                DemoState = BT_STATE_PROCESS; HciState=HCI_CMD_RESET;//YTS !!!!!!!!!!!!!!!!!
                UART2PrintString( "Generic demo device attached - event, deviceAddress=" );
                UART2PutDec( deviceAddress );
                UART2PrintString( "\r\n" );
                #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
                    for (i=1; i<((GENERIC_DEVICE_ID *)data)->serialNumberLength; i++)
                    {
                        UART2PutChar( ((GENERIC_DEVICE_ID *)data)->serialNumber[i] );
                    }
                #endif
                UART2PrintString( "\r\n" );
                return TRUE;
            }
            break;

        case EVENT_GENERIC_DETACH:
            deviceAddress   = 0;
            DemoState = BT_INITIALIZE;
            UART2PrintString( "Generic demo device detached - event\r\n" );
            return TRUE;

        case EVENT_GENERIC_TX_DONE:           // The main state machine will poll the driver.
        case EVENT_GENERIC_RX_DONE:
            return TRUE;

        case EVENT_VBUS_REQUEST_POWER:
            // We'll let anything attach.
            return TRUE;

        case EVENT_VBUS_RELEASE_POWER:
            // We aren't keeping track of power.
            return TRUE;

        case EVENT_HUB_ATTACH:
            UART2PrintString( "\r\n***** USB Error - hubs are not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            UART2PrintString( "\r\n***** USB Error - device is not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_CANNOT_ENUMERATE:
            UART2PrintString( "\r\n***** USB Error - cannot enumerate device *****\r\n" );
            return TRUE;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            UART2PrintString( "\r\n***** USB Error - client driver initialization error *****\r\n" );
            return TRUE;
            break;

        case EVENT_OUT_OF_MEMORY:
            UART2PrintString( "\r\n***** USB Error - out of heap memory *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
            UART2PrintString( "\r\n***** USB Error - unspecified *****\r\n" );
            return TRUE;
            break;

        case EVENT_SUSPEND:
        case EVENT_DETACH:
        case EVENT_RESUME:
        case EVENT_BUS_ERROR:
            return TRUE;
            break;

        default:
            break;
    }

    return FALSE;

} // USB_ApplicationEventHandler


//******************************************************************************
//******************************************************************************
// Main
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        main
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         Never exits
 *
 * Side Effects:    Runs the application
 *
 * Overview:        This is the USB Custom Demo Application's main entry
 *                  point.
 *************************************************************************/

int main ( void )
{

  // Initialize the processor and peripherals.
    if ( InitializeSystem() != TRUE )
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - system.  Halting.\r\n\r\n" );
        while (1);
    }
    if ( USBHostInit(0) == TRUE )
    {
        UART2PrintString( "\r\n\r\n***** USB Custom Demo App Initialized *****\r\n\r\n" );
    }
    else
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - USB.  Halting.\r\n\r\n" );
        while (1);
    }

    // Main Processing Loop
    while (1)
    {
        // This demo does not check for overcurrent conditions.  See the
        // USB Host - Data Logger for an example of overcurrent detection
        // with the PIC24F and the USB PICtail Plus.

        // Maintain USB Host State
        USBHostTasks();
        // Maintain Demo Application State
        ManageDemoState();
    }

    return 0;

} // main


/*************************************************************************
 * EOF main.c
 */

